/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.projecteer.metadata;

import biz.massivedynamics.projecteer.metadata.entity.Entity;
import biz.massivedynamics.projecteer.metadata.version.Version;

/**
 * A {@link Project} class that provides the following metadata:
 * 
 * <ul>
 *     <li>The project's name</li>
 *     <li>The project's description</li>
 *     <li>The name of the project's primary license</li>
 *     <li>The entity owning the project</li>
 * </ul>
 *
 * @author Cruz Julian Bishop
 * @param <V> The {@link Version} class to use
 * @since 1.0.0.0
 */
public class Project<V extends Version> implements Cloneable {
    
    /**
     * Creates a new {@link Project}
     * 
     * @param name This {@link Project}'s name
     * @param description This {@link Project}'s description
     * @param version The {@link Version} of this {@link Project}
     * @param licenseName The name of this {@link Project}'s license
     * @param owner The {@link Entity} that owns this {@link Project}
     * @since 1.0.0.0
     */
    public Project(String name, String description, V version, String licenseName, Entity owner) {
        this.name = name;
        this.description = description;
        this.version = version;
        this.licenseName = licenseName;
        this.owner = owner;
    }
    
    /**
     * This {@link Project}'s name
     * 
     * @since 1.0.0.0
     */
    private String name;
    
    /**
     * Gets this {@link Project}'s name
     * 
     * @return The name being used
     * @since 1.0.0.0
     */
    public String getName() {
        return this.name;
    }
    
    /**
     * Sets this {@link Project}'s name
     * 
     * @param name The new name to use
     * @since 1.0.0.0
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * This {@link Project}'s description.
     * <br />
     * A single sentence will usually do
     * 
     * @since 1.0.0.0
     */
    private String description;
    
    /**
     * Gets this {@link Project}'s description
     * 
     * @return The description being used
     * @since 1.0.0.0
     */
    public String getDescription() {
        return this.description;
    }
    
    /**
     * Sets this {@link Project}'s description
     * 
     * @param description The description to be used
     * @since 1.0.0.0
     */
    public void setDescription(String description) {
        this.description = description;
    }
    
    /**
     * The {@link Version} of this {@link Project}
     * 
     * @since 1.0.0.0
     */
    private V version;
    
    /**
     * Gets this {@link Project}'s {@link Version}
     * 
     * @return The {@link Version}
     * @since 1.0.0.0
     */
    public V getVersion() {
        return this.version;
    }
    
    /**
     * Sets this {@link Project}'s {@link Version}
     * 
     * @param version The {@link Version} to use
     * @since 1.0.0.0
     */
    public void setVersion(V version) {
        this.version = version;
    }
    
    /**
     * The name of this {@link Project}'s license
     * 
     * @since 1.0.0.0
     */
    private String licenseName;
    
    /**
     * Gets the name of this {@link Project}'s license
     * 
     * @return The name of the license
     * @since 1.0.0.0
     */
    public String getLicenseName() {
        return this.licenseName;
    }
    
    /**
     * Sets the name of this {@link Project}'s license
     * 
     * @param licenseName The license name
     * @since 1.0.0.0
     */
    public void setLicenseName(String licenseName) {
        this.licenseName = licenseName;
    }
    
    /**
     * The {@link Entity} that owns this {@link Project}
     * 
     * @since 1.0.0.0
     */
    private Entity owner;
    
    /**
     * Gets the {@link Entity} that owns this {@link Project}
     * 
     * @return The owner of this {@link Project}
     * @since 1.0.0.0
     */
    public Entity getOwner() {
        return this.owner;
    }
    
    /**
     * Sets the {@link Entity} that owns this {@link Project}
     * 
     * @param owner The new owner of this {@link Project}
     * @since 1.0.0.0
     */
    public void setOwner(Entity owner) {
        this.owner = owner;
    }
    
    /**
     * Clones this {@link Project}
     * 
     * @return The cloned {@link Project}
     * @since 1.0.0.0
     */
    @Override
    public Project clone() {
        return new Project(this.name, this.description, this.version, this.licenseName, this.getOwner().clone());
    }
    
}
