/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.projecteer.metadata.entity;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * A {@link Group} is an {@link Entity} that contains a list of other Entities
 *
 * @author Cruz Julian Bishop
 * @since 1.0.0.0
 */
public class Group implements Entity {
    
    /**
     * Creates a new {@link Group}
     * 
     * @param name The name to use
     * @param role The role of this {@link Group}
     * @param emailAddress This {@link Group}'s email address
     * @param members The {@link Entity} members of this group
     * @since 1.0.0.0
     */
    public Group(String name, String role, String emailAddress, Entity... members) {
        this.name = name;
        this.role = role;
        this.emailAddress = emailAddress;
        this.members = new ArrayList(Arrays.asList(members));
    }
    
    /**
     * The name of this {@link Group}
     * 
     * @since 1.0.0.0
     */
    private String name;

    /**
     * Gets the name of this {@link Group}
     * 
     * @return The name
     * @since 1.0.0.0
     */
    @Override
    public String getName() {
        return this.name;
    }

    /**
     * Gives this {@link Group} a new name
     * 
     * @param name The new name to use
     * @since 1.0.0.0
     */
    @Override
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * The role of this {@link Group}
     * 
     * @since 1.0.0.0
     */
    private String role;

    /**
     * Gets the role of this {@link Group}
     * 
     * @return The role
     * @since 1.0.0.0
     */
    @Override
    public String getRole() {
        return role;
    }

    /**
     * Gives this {@link Group} a new role
     * 
     * @param role The {@link Group}'s new role
     * @since 1.0.0.0
     */
    @Override
    public void setRole(String role) {
        this.role = role;
    }
    
    /**
     * The email address for this {@link Group}
     * 
     * @since 1.0.0.0
     */
    private String emailAddress;

    /**
     * Gets this {@link Group}'s email address
     * 
     * @return The {@link Group}'s email address
     * @since 1.0.0.0
     */
    @Override
    public String getEmailAddress() {
        return this.emailAddress;
    }

    /**
     * Gives this {@link Group} a new email address
     * 
     * @param emailAddress The email address to use
     * @since 1.0.0.0
     */
    @Override
    public void setEmailAddress(String emailAddress) {
        this.emailAddress = emailAddress;
    }
    
    /**
     * The {@link List} of {@link Entity} members
     * 
     * @since 1.0.0.0
     */
    private List<Entity> members;
    
    /**
     * Gets the {@link List} of {@link Entity} members
     * 
     * @return The {@link List} of members
     * @since 1.0.0.0
     */
    public List<Entity> getMembers() {
        return this.members;
    }
    
    /**
     * Creates a {@link String}-based representation of this {@link Group}
     * 
     * @return A {@link String} representing this {@link Group}
     * @since 1.0.0.0
     */
    @Override
    public String toString() {
        return this.name + " (" + this.members.size() + " members) <" + this.emailAddress + ">";
    }
    
    /**
     * Checks to see if this {@link Group} is equal to another object, nominally a {@link Group}
     * 
     * @param other The other {@link Group}
     * @return True if equal, otherwise false
     * @since 1.0.0.0
     */
    @Override
    public boolean equals(Object other) {
        if (other instanceof Group) {
            Group otherGroup = (Group) other;
            
            //Check the hash code
            if (otherGroup.hashCode() != this.hashCode()) return false;
            
            //Check the name
            if (!otherGroup.getName().equals(this.getName())) return false;
            
            //Check the email
            if (!otherGroup.getEmailAddress().equals(this.getEmailAddress())) return false;
            
            //Check the members
            if (!otherGroup.getMembers().equals(this.getMembers())) return false;
            
            return true;
            
        } else return false;
    }
    
    /**
     * Gets the hash code for this {@link Group}
     * 
     * @return The hash code
     * @since 1.0.0.0
     */
    @Override
    public int hashCode() {
        int hash = this.name.hashCode() << 16 | this.emailAddress.hashCode() << 8;
        
        for (Entity entity : this.members) {
            hash |= entity.hashCode();
        }
        
        return hash;
    }
    
    /**
     * Clones this {@link Group} and all of it's members
     * 
     * @return The cloned {@link Group}
     * @since 1.0.0.0
     */
    @Override
    public Group clone() {
        Group group = new Group(this.name, this.role, this.emailAddress);
        
        for (Entity member : this.members) {
            group.members.add(member.clone());
        }
        
        return group;
    }
    
}
