/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.projecteer.metadata.entity;

/**
 * An {@link Organization} is essentially an extended {@link Group} that is
 * usually created as a registered business
 *
 * @author Cruz Julian Bishop
 * @since 1.0.0.0
 */
public class Organization extends Group {
    
    /**
     * Creates a new {@link Organization}
     * 
     * @param name The {@link Organization}'s name
     * @param role The {@link Organization}'s role
     * @param emailAddress The {@link Organization}'s email address
     * @param members The members of this {@link Organization}
     * @since 1.0.0.0
     */
    public Organization(String name, String role, String emailAddress, Entity... members) {
        super(name, role, emailAddress, members);
        
        this.registeredBusiness = false;
    }
    
    /**
     * Creates a new {@link Organization}
     * 
     * @param name The {@link Organization}'s name
     * @param role The {@link Organization}'s role
     * @param emailAddress The {@link Organization}'s email address
     * @param registeredBusiness True if this {@link Organization} is a registered business
     * @param members The members of this {@link Organization}
     * @since 1.0.0.0
     */
    public Organization(String name, String role, String emailAddress, boolean registeredBusiness, Entity... members) {
        super(name, role, emailAddress, members);
        
        this.registeredBusiness = registeredBusiness;
    }
    
    /**
     * Checks to see if this {@link Organization} is a registered business.
     * 
     * @since 1.0.0.0
     */
    private boolean registeredBusiness;
    
    /**
     * Checks to see if this {@link Organization} is a registered business
     * 
     * @return True if this organization is a registered business, otherwise false
     * @since 1.0.0.0
     */
    public boolean isRegisteredBusiness() {
        return this.registeredBusiness;
    }
    
    /**
     * Sets this organization as a registered business.
     * <br />
     * Please note that this does not actually make it legally registered.
     * You have to do that yourself!
     * 
     * @since 1.0.0.0
     */
    public void registerAsBusiness() {
        this.registeredBusiness = true;
    }
    
    /**
     * Sets this organization as a unregistered business.
     * <br />
     * Please note that this does not actually make it legally unregistered.
     * You have to do that yourself!
     * 
     * @since 1.0.0.0
     */
    public void unregisterAsBusiness() {
        this.registeredBusiness = false;
    }
    
    /**
     * Clones this {@link Organization} and all of it's members
     * 
     * @return The cloned {@link Organization}
     * @since 1.0.0.0
     */
    @Override
    public Organization clone() {
        Organization organization = new Organization(this.getName(), this.getRole(),
                this.getEmailAddress(), this.isRegisteredBusiness());
        
        for (Entity member : this.getMembers()) {
            organization.getMembers().add(member.clone());
        }
        
        return organization;
    }
    
    
}
