/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package biz.massivedynamics.projecteer.metadata.entity;

/**
 * A {@link Person} who is an {@link Entity} that is linked to the current project
 *
 * @author Cruz Julian Bishop
 * @since 1.0.0.0
 */
public class Person implements Entity {
    
    /**
     * Creates a new anonymous {@link Person}
     * 
     * @since 1.0.0.0
     */
    public Person() {
        this("Anonymous");
    }
    
    /**
     * Creates a new {@link Person} with an unspecified role and email address
     * 
     * @param name The {@link Person}'s name
     * @since 1.0.0.0
     */
    public Person(String name) {
        this(name, "Unknown");
    }
    
    /**
     * Creates a new {@link Person} with an unspecified email address
     * 
     * @param name The {@link Person}'s name
     * @param role The {@link Person}'s role
     * @since 1.0.0.0
     */
    public Person(String name, String role) {
        this(name, role, "unknown@null.com");
    }
    
    /**
     * Creates a new {@link Person}
     * 
     * @param name The {@link Person}'s name
     * @param role The {@link Person}'s role
     * @param emailAddress The {@link Person}'s email address
     * @since 1.0.0.0
     */
    public Person(String name, String role, String emailAddress) {
        this.name = name;
        this.role = role;
        this.emailAddress = emailAddress;
    }
    
    /**
     * This {@link Person}'s name
     * 
     * @since 1.0.0.0
     */
    private String name;
    
    /**
     * This {@link Person}'s role
     * 
     * @since 1.0.0.0
     */
    private String role;
    
    /**
     * This {@link Person}'s email address
     * 
     * @since 1.0.0.0
     */
    private String emailAddress;

    /**
     * Gets this {@link Person}'s name
     * 
     * @return The name
     * @since 1.0.0.0
     */
    @Override
    public String getName() {
        return this.name;
    }
    
    /**
     * Sets this {@link Person}'s name
     * 
     * @param name The new name to use
     * @since 1.0.0.0
     */
    @Override
    public void setName(String name) {
        this.name = name;
    }

    /**
     * Gets this {@link Person}'s role
     * 
     * @return The role
     * @since 1.0.0.0
     */
    @Override
    public String getRole() {
        return this.role;
    }
    
    /**
     * Sets this {@link Person}'s role
     * 
     * @param role The role to use
     * @since 1.0.0.0
     */
    @Override
    public void setRole(String role) {
        this.role = role;
    }

    /**
     * Gets this {@link Person}'s email address
     * 
     * @return The email address
     * @since 1.0.0.0
     */
    @Override
    public String getEmailAddress() {
        return this.emailAddress;
    }
    
    /**
     * Sets this {@link Person}'s email address
     * 
     * @param emailAddress The email address to set
     * @since 1.0.0.0
     */
    @Override
    public void setEmailAddress(String emailAddress) {
        this.emailAddress = emailAddress;
    }
    
    /**
     * Checks to see if this {@link Person} is equal to another {@link Person}
     * 
     * @param other The other {@link Object}, which is nominally a {@link Person}
     * @return True if equal, otherwise false
     * @since 1.0.0.0
     */
    @Override
    public boolean equals(Object other) {
        //First, check to see if it's a person
        if (other instanceof Person) {
            
            //It's a person. Set up a temporary person
            Person otherPerson = (Person) other;
            
            //Check the hash code
            if (otherPerson.hashCode() != this.hashCode()) return false;
            
            //Check the name
            if (!otherPerson.getName().equals(this.getName())) return false;
            
            //Check the email
            if (!otherPerson.getEmailAddress().equals(this.getEmailAddress())) return false;
            
            return true;
            
        } else return false;
        
    }
    
    /**
     * Generates the hash code for this {@link Person}
     * 
     * Does not include role, since it can change on a regular basis across projects
     * 
     * @return The hash code
     * @since 1.0.0.0
     */
    @Override
    public int hashCode() {
        return this.name.hashCode() << 8 | this.emailAddress.hashCode();
    }
    
    /**
     * Provides a {@link String}-based representation of this {@link Person}
     * 
     * @return A {@link String} representing this {@link Person}
     * @since 1.0.0.0
     */
    @Override
    public String toString() {
        return this.name + " <" + this.emailAddress + ">";
    }
    
    /**
     * Clones this {@link Person}
     * 
     * @return The new person
     * @since 1.0.0.0
     */
    @Override
    public Person clone() {
        return new Person(this.name, this.role, this.emailAddress);
    }
    
}
